function IE = FDI_isolate_estimate(~, e, detect)

%FDI_ISOLATE_ESTIMATE is the function for the isolation & estimation stage
%of the FDI filter.
%   This file filters the aileron deflection error signal and analyzes its
%   steady-state value. In addition, the rudder deflection error signal is
%   filtered to obtain an estimate of the fault.
%   Input definition:
%   ~       = thresholds
%   e       = residuals
%   detect  = detection structure (output of FDI_detect)

%% Check if fault has been detected
if max(abs(detect.global)) ~= 1
    IE.terminate = 'No fault has been detected. Isolation is possible only after fault detection.\n';
    return; % error protection
end

%% Low pass filter definition
% A low-pass filter has transfer function
% H(s) = K/(1 + tau*s)
% K is the passband gain
% (1/tau) is the cutoff frequency in rad/s
tau = 2; % filter time constant
K = 1; % passband gain
lp_filter = tf(K, [tau, 1]);
figure; bode(lp_filter); grid on; title('Bode plot of the low-pass transfer function');

%% Low pass filter the aileron deflection command
[IE.lpf.rail, IE.lpf.time] = lsim(lp_filter, e.rail, e.time);

%% Low pass filter the rudder deflection command
[IE.lpf.rud, IE.lpf.time] = lsim(lp_filter, e.rud, e.time);

%% Declare steady-state for aileron
% Steady-state of the aileron deflection command is declared when the
% standard deviation of the filtered signal within a specified interval is
% below a specified limit.
IE.spec.interval = 50; % number of frames in the specified interval
IE.spec.mean = 0.35; % mean specification
IE.spec.range = 0.015; % range specification
IE.spec.std = 0.005; % standard deviation specification
Ktc = 35; % lat-dir controller settling time (=3*time_constant)
IE.spec.hittime = detect.hittime.global + Ktc;
ind = find(e.time == detect.hittime.global, 1, 'first'); % index of the global hitting time

for k = ind+IE.spec.interval:numel(IE.lpf.time)
    mean_rail = mean(IE.lpf.rail(k-IE.spec.interval:k));
    range_rail = range(IE.lpf.rail(k-IE.spec.interval:k));
    std_rail = std(IE.lpf.rail(k-IE.spec.interval:k));
    if      (abs(mean_rail) < IE.spec.mean ...
            && range_rail < IE.spec.range ...
            && std_rail < IE.spec.std ...
            && IE.lpf.time(k) > IE.spec.hittime) % check if all specs are satisfied
        IE.hitind = k;
        IE.range = range_rail;
        IE.std = std_rail;
        IE.est.rail = IE.lpf.rail(k);
        break;
    else
        continue;
    end
end

%% Steady state rudder command
IE.hittime = IE.lpf.time(IE.hitind);
IE.est.rud = IE.lpf.rud(IE.hitind);

%% Isolation and estimation of the fault using isolation matrix
% The fault isolation matrix is encoded in these if/else constructs.
if (IE.est.rud < 0 && IE.est.rail > 0) || (IE.est.rud > 0 && IE.est.rail < 0)
    IE.source = 'Top';
elseif (IE.est.rud > 0 && IE.est.rail > 0) || (IE.est.rud < 0 && IE.est.rail < 0)
    IE.source = 'Bottom';
else
    IE.source = 'unresolved';
end

end