%% FDI filter plotting
% This file generates the plots for \subsection{Results\Fault level} for
% the paper submitted to AIAA SciTech 2015.

%% Clean the slate
clear all, close all, bdclose all, clc

%% Read files from disk
% Sweep over -25deg to +25deg for stuck rudder faults.
fault_vec = -25:+25;

% Top rudder faults
for k = 1:numel(fault_vec)
    clearvars -except fault_vec k fldata
    whos
    path = input('Enter the path: ');
    t_rud_fault = fault_vec(k); % set top rudder fault level
    t_rud_inject_time = 5; % set top rudder fault injection time
    b_rud_fault = 0; % set bottom rudder fault level
    b_rud_inject_time = 1000; % set bottom rudder fault injection time
    faultname = strrep(num2str(t_rud_fault, '%4.2f'), '.', '_');
    filename = ['simres_SIL_Ft', faultname, '_cz'];
    load([path, filename, '.mat']);
    load([path, 'simres_L_Uf_cz_100s', '.mat']);
    run('FDI_filter_perf.m'); % run FDI filter performance analyzer
    fldata.top.fl(k) = t_rud_fault;
    try % set detection time, estimation time, and fault estimate
        fldata.top.dettime(k) = detect.hittime.global - t_rud_inject_time;
        fldata.top.IEtime(k) = IE.hittime - detect.hittime.global;
        fldata.top.est(k) = -1*IE.est.rud;
    catch % catch errors if fault was not detected
        fldata.top.dettime(k) = NaN;
        fldata.top.IEtime(k) = NaN;
        fldata.top.est(k) = 0;
    end
end

% Bottom rudder faults
for k = 1:numel(fault_vec)
    clearvars -except fault_vec k fldata
    whos
    path = input('Enter the path: ');
    t_rud_fault = 0; % set top rudder fault level
    t_rud_inject_time = 1000; % set top rudder fault injection time
    b_rud_fault = fault_vec(k); % set bottom rudder fault level
    b_rud_inject_time = 5; % set bottom rudder fault injection time
    faultname = strrep(num2str(b_rud_fault, '%4.2f'), '.', '_');
    filename = ['simres_SIL_Fb', faultname, '_cz'];
    load([path, filename, '.mat']);
    load([path, 'simres_L_Uf_cz_100s', '.mat']);
    run('FDI_filter_perf.m'); % run FDI filter performance analyzer
    fldata.bottom.fl(k) = b_rud_fault;
    try % set detection time, estimation time, and fault estimate
        fldata.bottom.dettime(k) = detect.hittime.global - b_rud_inject_time;
        fldata.bottom.IEtime(k) = IE.hittime - detect.hittime.global;
        fldata.bottom.est(k) = -1*IE.est.rud;
    catch % catch errors if fault was not detected
        fldata.bottom.dettime(k) = NaN;
        fldata.bottom.IEtime(k) = NaN;
        fldata.bottom.est(k) = 0;
    end
end

clearvars -except fault_vec k fldata

%% Plotting the results

% Detection time limit markers
dtlim.flneg = [-7; -7];
dtlim.flpos = [7; 7];
dtlim.ycoord = [-1e3, 1e3];

% Plots
figure;

subplot(121); % detection time
hold on;
plot(dtlim.flneg, dtlim.ycoord, 'm--', 'LineWidth', 2);
plot(dtlim.flpos, dtlim.ycoord, 'm--', 'LineWidth', 2);
ftdet = plot(fldata.top.fl, fldata.top.dettime, 'b.-', 'MarkerSize', 8, 'LineWidth', 2);
fbdet = plot(fldata.bottom.fl, fldata.bottom.dettime, 'r--+', 'MarkerSize', 6, 'LineWidth', 2);
hold off;
grid on;
xlabel('Injected fault [deg]', 'FontSize', 14, 'FontWeight', 'bold');
ylabel('Detection time [s]', 'FontSize', 14, 'FontWeight', 'bold');
a1 = gca;
set(a1, 'XMinorTick', 'on', 'YMinorTick', 'on', ...
        'XLim', [-25 25], 'YLim', [0.2 0.8], ...
        'XTick', [-25, -15, 0, 15, 25], 'YTick', 0.2:0.2:0.8, ...
        'FontSize', 14, 'FontWeight', 'bold');
L1 = legend([ftdet, fbdet], 'Top', 'Bottom', 'Location', 'Best');
set(L1, 'FontSize', 14, 'FontWeight', 'bold');
udf = text(0, 0.6, 'Undetectable fault');
set(udf, 'rotation', 90, ...
    'HorizontalAlignment', 'center',...
    'BackgroundColor', [1 1 1], ...
    'EdgeColor', [0 0 0], ...
    'FontSize', 14, 'FontWeight', 'bold');

subplot(122); % fault estimate
hold on;
plot(dtlim.flneg, dtlim.ycoord, 'm--', 'LineWidth', 2);
plot(dtlim.flpos, dtlim.ycoord, 'm--', 'LineWidth', 2);
ftest = plot(fldata.top.fl, fldata.top.est, 'b.-', 'MarkerSize', 8, 'LineWidth', 2);
fbest = plot(fldata.bottom.fl, fldata.bottom.est, 'r--+', 'MarkerSize', 6, 'LineWidth', 2);
hold off;
grid on;
xlabel('Injected fault [deg]', 'FontSize', 14, 'FontWeight', 'bold');
ylabel('Fault estimate [deg]', 'FontSize', 14, 'FontWeight', 'bold');
a1 = gca;
set(a1, 'XMinorTick', 'on', 'YMinorTick', 'on', ...
        'XLim', [-25 25], 'YLim', [-25 25], ...
        'XTick', [-25, -15, 0, 15, 25], 'YTick', [-25, -15, 0, 15, 25], ...
        'FontSize', 14, 'FontWeight', 'bold');
L1 = legend([ftest, fbest], 'Top', 'Bottom', 'Location', 'Best');
set(L1, 'FontSize', 14, 'FontWeight', 'bold');
udf = text(0, 12, 'Undetectable fault');
set(udf, 'rotation', 90, ...
    'HorizontalAlignment', 'center',...
    'BackgroundColor', [1 1 1], ...
    'EdgeColor', [0 0 0], ...
    'FontSize', 14, 'FontWeight', 'bold');
